#!/usr/bin/perl
# Erwin.pl
# Author: Alex Brasetvik <alex at brasetvik dot com>
# Author: Idar Lund <effbiai at users dot sf dot net>
# $Id$
#
# Read the README for usage information.
# Short overview of commands available with Erwin.pl --help

use POE qw(Component::IRC::State);

use lib "lib";
use lib "lib/Modules";

use Core;
use Common;
use Plugin;
use Digest::MD5 qw(md5_base64);
use My::DBI qw(:all);
use Getopt::Long;
use Term::ReadKey;

use Data::Dumper;

use strict;
use warnings;
#use diagnostics;
use vars qw($core %config $VERSION);

use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', 'locale/');
textdomain('erwin');

$VERSION = '1.2.0';

my $config = {};
my @configs = ('config.defaults');

my ($username, $password, $flags, $active);
$active = 1;

#Read command line options overriding configuration settings.
GetOptions(
        #General
        'nick|n|nickname:s', \$config->{Nick},
        'conf|f:s', \$configs[1],
        'pidfile:s', \$config->{PidFile},
        'stdout!', \$config->{LogStdout},
        'try|t', sub { Try(); },
        #Add user
        'password:s', \$password,
        'username:s', \$username,
        'flags:s', \$flags,
        'active!', \$active,
        'adduser', sub { AddUser(); },
        #Help
        'help|h', sub { Help(); },
        #List users
        #'list-users', sub { ListUsers(); },
);

die "Do not run as root!" unless($<);
unless($configs[1]) {
    if( -r "conf.pl" ) {
        $configs[1] = 'conf.pl';
    } else {
        print gettext("Please specify a configuration file.\n");
        exit(1);
    }
}

print replace(gettext("Can't read \"\$FILE\": \$ERROR\n"), { FILE => $configs[1], ERROR => $! }) and exit(1) unless(-r $configs[1]);

DoConfig();
$config->{ConfigFiles} = \@configs;

if(-e $config->{PidFile}) {
    print STDERR gettext("PID file exists. Stale PID file or already running?\n");
    #exit 1;
}

$SIG{HUP} = sub {
    $core->Rehash;
};

if(!$config->{LogStdout}) {
    exit 0 if fork();
    open( FILE, '>' . $config->{PidFile} ) or die gettext("Couldn't write PID file:") . " $!";
    print FILE $$;
    close(FILE);
    close(STDOUT);
}

# The core package takes care of reloading modules, regexes, etc and stores $dbh
$core = new Core($config);

#my $erwin = POE::Component::IRC->spawn();
my $erwin = POE::Component::IRC::State->spawn( %$config ) or die "WTF $!";
$core->setErwin($erwin);

# Initiate something that runs periodically
POE::Session->create(
        package_states => [
        'main' => [qw(_start _stop irc_disconnected irc_socketerr irc_error irc_public irc_msg irc_join irc_352 irc_nick irc_mode irc_kick irc_quit irc_part irc_001 irc_433 irc_005 irc_invite)]
        ],
        heap => {
        erwin => $erwin,
        },
);

# TODO: Not working yet.
=cut
POE::Session->create(
        inline_states => {
        _start => sub {
        $_[KERNEL]->alarm( tick => time() + 1 );
        },
        tick => sub {
        $core->cron;
        $_[KERNEL]->alarm( tick => time() + 1 );
        },
        },
        );
=cut

# run the damn thing :-)
$poe_kernel->run();

unlink $core->config->{PidFile} if( -w $core->config->{PidFile} );
exit 0;

sub _start {
    my $kernel = $_[KERNEL];
    my $heap = $_[HEAP];

    $core->setKernel($kernel);
    $core->setHeap($heap);

    $erwin->yield( register => 'all' );
    $erwin->yield( connect  => $core->config );
    #$kernel->post( erwin => register => 'all' );

    #$kernel->post( erwin => connect => $core->config );
    #$kernel->post( erwin => connect => {} );


}

#On connect
sub irc_001
{
    my $kernel = $_[KERNEL];
    my $heap = $_[HEAP];

    $core->log(replace(gettext('Connected to $SERVER.'), { SERVER => $core->config->{Server}}), NORMAL);
    #my $erwin = $heap->{erwin};
    #%{ $core->hostmasks } = (); #Clear old hostmasks.
    foreach my $channel ( @{ $core->config->{Channel} } ) {
    #$kernel->post( 'erwin', 'join', $channel );
        $erwin->yield(join => $channel);
        $core->log(replace(gettext('Joining "$CHANNEL".'), { CHANNEL => $channel }), NORMAL);
        push(@{$core->channels}, $channel);
    }

}

#XXX - Should reconnect.
sub irc_disconnected {
    my ($server) = $_[ARG0];
    $core->log(replace(gettext('Lost connection to "$SERVER".'), { SERVER => $server }), ERROR);
    #Should reconnect.
    exit;
}

sub irc_error {
    my $err = $_[ARG0];
    $core->log(replace(gettext('Server error: $ERROR'), { ERROR => $err }), ERROR);
}

sub irc_socketerr {
    my $err = $_[ARG0];
    $core->log(replace(gettext('Couldn\'t connect: $ERROR'), { ERROR => $err }), CRISIS);
}

sub _stop {
    my ($kernel) = $_[KERNEL];
    $core->log(gettext('Controll session stopped'), NORMAL);
    $kernel->yield(quit => 'Leaving' );
}

sub irc_public {
    my ( $kernel, $who, $where, $msg ) = @_[ KERNEL, ARG0 .. ARG2 ];

    $where = $where->[0];
    $core->heap->{where} = $where;
    $core->heap->{who} = $who;
    $core->heap->{nick} = substr( $who, 0, index( $who, "!" ) );
    $core->heap->{msg} = $msg;
    $core->heap->{public} = 1;
    $core->heap->{privmsg} = 0;
    $core->message;
    $core->reset;
}

sub irc_join {
    my ( $kernel, $who, $where) = @_[ KERNEL, ARG0 .. ARG1 ];

    my $nick = substr($who,0,index($who,"!"));

    if($nick eq $core->mynick) {
    #I am joining. Fetch a list of nicks and hosts. See irc_352()
        $erwin->yield(who => $where);
    } else {
        $core->log(replace(gettext('$WHO joined $WHERE'), { WHO => $who, WHERE => $where }), INFO);
        $core->addhost($who, $where);
    }

    $core->join($who, $where);
    $core->reset;
}

sub irc_invite {
    my ( $kernel, $who, $channel) = @_[ KERNEL, ARG0 .. ARG1 ];
    $core->log(replace(gettext('$WHO invited me to $CHANNEL'), { WHO => $who, CHANNEL => $channel }), INFO);
    if ($channel ~~ @{$core->config->{Channel}}) {
        $core->log(replace(gettext('joining $CHANNEL'), { CHANNEL => $channel }), INFO);
        $erwin->yield(join => $channel);
    } else {
        $core->log(replace(gettext('$CHANNEL not in channel list'), { CHANNEL => $channel }), INFO);
        return 0;
    }
}

# /WHO -- giving a list of users with channel, ident, host, etc..
# Called once for every user.
# One can always get the nick out of a hostmask, but not vice versa, and only the nick is available
# upon mode changes, kicks, etc., and it's handy to be able to get the hostmask then.
sub irc_352
{
    my ($kernel,undef,$string) = @_[ KERNEL, ARG0 .. ARG1 ];
    my ($channel, $ident, $host, $server, $nick, $garbage) = split(/ /, $string, 6);
    my $who = "$nick!$ident\@$host";

    $core->log(replace(gettext('Setting hostmask to "$HOSTMASK" for $NICK.'), { HOSTMASK => $who, NICK => $nick }), DEBUG);
    $core->addhost($who, $channel);
    $core->who($who, $channel);
}

#RPL_ISUPPORT, see http://www.irc.org/tech_docs/005.html
#e.g. MODES=4 MAXCHANNELS=50 MAXBANS=100 MAXTARGETS=4 NICKLEN=9 TOPICLEN=120 KICKLEN=120
sub irc_005
{
    my $isupport = $_[ARG1];

    foreach my $key (qw(MODES MAXCHANNELS MAXBANS MAXTARGETS NICKLEN TOPICLEN KICKLEN)) {
        if($isupport =~ /$key=(\S+)/) {
            $core->serverinfo->{$key} = $1;
        }
    }
}

#ERR_NICKNAMEINUSE
sub irc_433
{
    $core->log(replace(gettext('Nickname $NICK in use.'), { NICK => $core->mynick }), ERROR);
    $core->mynick =~ /^(.*)(.{1})$/;
    $core->Nick("$2$1");
}

sub irc_mode
{
    my ($kernel,$bywhom,$where,$mode,@vars) = @_[ KERNEL, ARG0 .. $#_];
    #   $core->log(replace(gettext('Mode $MODE$TOWHOM on $WHERE by $BYWHOM '),
    #      { MODE => $mode, TOWHOM => $towhom, WHERE => $where, BYWHOM => $bywhom }), INFO);
    $core->mode($bywhom, $where, $mode, @vars);
    $core->reset;
}

sub irc_kick
{
    my ($kernel,$kicker,$where,$kicked, $kickmsg) = @_[ KERNEL, ARG0 .. ARG3 ];
    $core->log(replace(gettext('$KICKED kicked $KICKED from $WHERE'), { KICKER => $kicker, WHERE => $where, KICKED => $kicked }), INFO);
    $core->kick($kicker, $where, $kicked, $kickmsg);
    $core->removehost($core->getHost($kicked), $where);
    $core->reset;
}

sub irc_quit
{
    my ($kernel,$who, $msg) = @_[ KERNEL, ARG0, ARG1 ];
    $core->quit($who, $msg);
    $core->removehost($who);
    $core->log(replace(gettext('$WHO quit.'), { WHO => $who }), INFO);
}

sub irc_part
{
    my ($kernel,$who,$where) = @_[ KERNEL, ARG0 .. ARG1 ];
    $core->part($who, $where);
    $core->removehost($who, $where);
    $core->log(replace(gettext('$WHO left $WHERE.'), { WHO => $who, WHERE => $where }), INFO);
}

sub irc_nick
{
    my ($kernel,$who,$newnick) = @_[ KERNEL, ARG0 .. ARG1 ];

    return if($newnick eq $core->mynick);

    my $oldnick = substr($who,0,index($who,"!"));

    $core->nick($who, $newnick);
}

sub irc_msg {
    my ( $kernel, $who, $where, $msg ) = @_[ KERNEL, ARG0 .. ARG2 ];
    $core->heap->{who} = $who;
    $core->heap->{nick} = substr( $who, 0, index( $who, "!" ) );
    $core->heap->{where} = $core->heap->{nick};
    $core->heap->{msg} = $msg;
    $core->heap->{public} = 0;
    $core->heap->{privmsg} = 1;
    $core->message;
    $core->reset;
}
sub Help
{
    print replace(gettext("
        This is Erwin, \$VERSION.

        Command line options:

        General configuration options:
        \t--conf -f\t\tConfiguration file. (Default: conf.pl)
        \t--channel -c\t\tJoin this channel. Specify multiple channels with -c #chan1 -c #chan2.
        \t--pidfile\t\tFile to write PID to.

        Debug:
        \t--try -t\t\tVerify configuration, modules and regexes without connecting.
        \t--stdout\t\tLog to stdout and do not fork.

        Misc.:
        \t--help -h\t\tThis message.
    "), { VERSION => $VERSION });

    exit 0;
}

sub Try
{
    $config = {};

    unless($configs[1]) {
        if( -r "conf.pl" ) {
            $configs[1] = 'conf.pl';
        }
    }

    print gettext("Please specify a configuration file.\n") and exit(1) unless($configs[1]);
    print replace(gettext("Can't read \"\$FILE\": \$ERROR\n"), { FILE => $configs[1], ERROR => $! }) and exit(1) unless(-r $configs[1]);

    DoConfig();

    $config->{ConfigFiles} = \@configs;
    $config->{LogStdout} = 1;
    my $core = new Core($config);
    exit 0;
}

sub DoConfig
{
    my %temp;
    OUTER: for(@configs) {
        Core::load_config($core, $_ , $config);
        if(ref($config->{ConfigFiles}) eq 'ARRAY') {
            INNER: foreach my $conf (@{ $config->{ConfigFiles} }) {
                if($temp{$conf}) {
                    next INNER;
                } else {
                    push(@configs, $conf);
                    $temp{$conf} = 1;
                }
            }
        }
    }

}
