package Boggle;

use lib '../';
use warnings;
use strict;

use vars qw($core);

use My::DBI qw(:all);
use DBI;
use DBD::Pg;

use Boggle::Board;

use base qw(Plugin);

#States:
#a round is started and going on.
use constant B_STARTING => 1;
use constant B_RUNNING => 2;

#Start a new round
sub Start
{
   my($core, $backrefs, $args) = @_;
   
   if($core->heap->{privmsg}) { #In case of a badly configured regex.
      $core->log('Badly configured regex! Boggle::Start enabled for private messages.', WARNING);
      return;
   }
   
#   return unless(Quiz::User->GetByHostmask($core->dbh('quiz'), $core->heap->{who})); #Only logged in users are allowed to start.

   $config{core} = $core;
   $config{where} = $core->where;

   my $B = $core->heap->{Boggle}->{$core->where};

   return if($B && $B->state > 0); #Already running.

   $B = Boggle::Round->new(\%config);

   $core->heap->{Boggle}->{$core->where} = $B;
   $B->start;
}

sub cron
{
   my $core = shift;

   foreach my $B (values %{ $core->heap->{Boggle} }) {
      if($B->state == B_RUNNING) {
	 #Round is over.
	 $B->end;
      }
   }
   return;
}

#Run on every incoming message.
sub public
{
   my($core, $who, $where, $msg) = @_;

   return if($msg =~ /:/);

   $who =~ /(.*)!/;
   return if($1 eq $core->mynick);


   my $R = $core->heap->{Q}->{$where};
   return unless $R;

=cut
   my $stop_regex = gettext('^!q stopp?$');

   if($msg =~ /$stop_regex/i) {
      my $R = $core->heap->{Q}->{$core->where};
      $R->end;
      delete $core->heap->{Q}->{$core->where};
      return;
   }
=cut

   my $comment_regex = gettext('!q ?comment (\d+) (.+)');
   if($msg =~ /$comment_regex/) {
      $core->msg($where, $core->heap->{nick} . ": " . Quiz::Comment($core, [$1, $2], {}));
      return;
   }

   $R->ping;

   if($R->state == Q_GATHER) {
      # TODO - Add player to team.
   } elsif ($R->state == Q_QUESTION) {
      #A questions is asked, and the user wants to answer:
     
      my $User = $R->getUser($who);

      if($R->question->check_answer($msg)) {

         #my $User = Quiz::User->GetByHostmask($R->dbh, $who);

	 my $score = dbi_exec_for_scalar($core->dbh('quiz'), "SELECT reduced_question_score(?,?)", $R->question->id, $User->id);
	 prepare_and_exec($R->dbh, "UPDATE round_participants SET attempts = attempts + 1, wins = wins + 1 WHERE round=? AND user_id=?", $R->id, $User->id);
	 $R->askedQuestion($User);

         my $a = $R->question->format_answer;

	 my $TRESHOLD = 10; # XXX
	 my $count = dbi_exec_for_scalar($core->dbh('quiz'), "SELECT count(*) FROM round_questions WHERE round = ? AND winner = ?", $R->id, $User->id);

	 $R->msg(sprintf("Riktig svar: %s. (%s, %i/%i, +%.02fp)", $a, $User->nick, $count, $TRESHOLD, $score));

	 if( $count >= $TRESHOLD ) {
	    $R->end( $User );
            $core->setMode($core->where, '-m');
	 } else {
	    $R->state(Q_PAUSE);
	 }
      } else {
	 prepare_and_exec($R->dbh, "UPDATE round_participants SET attempts = attempts + 1 WHERE round=? AND user_id=?", $R->id, $User->id);
      }
   } else {
     return; #?
   }
}

sub nick
{
   my($core, $who, $newnick) = @_;

   $who =~ /(.*)!/;
   return if($1 eq $core->mynick);

   printf "*** In nick() in Quiz.pm: $who, $newnick\n";

   foreach my $where (keys %{ $core->heap->{Q} }) {
      my $R = $core->heap->{Q}->{$where};
      next unless $R;
      printf "*** Calling R->moveUser()\n";
      $R->moveUser($who, $newnick);
   }

   $who =~ /!(.*)/;
   my $newhostmask = "$newnick!$1";
   #XXX : Configurability.
   printf "*** Updating users-table.\n";
   prepare_and_exec($core->dbh('quiz'), 'UPDATE users SET hostmask=? WHERE hostmask=?', $newhostmask, $who);

}

sub join
{
   my($core, $who, $where) = @_;

   print "*** in join(); \n";

   my %config;
   $core->module_config(\%config, $where);
   
   print "*** enabled? \n";
   return unless $config{enabled};
   print "*** yes \n";

   my $User = Quiz::User->GetByHostmask($core->dbh('quiz'), $who);
   $who =~ /(.*)!/; my $nick = $1;
   
   if($User) {
      $core->setMode($where,'+v', $nick);
      $core->msg($nick, sprintf("Hei. Du er logget inn som \"%s\".", $User->username));
   } else {
      $core->msg($nick, "Hei. Du er ikke logget inn. For å spille må du ha en bruker og være logget inn. Skriv " . BOLD . "!q auth" . RESET .
	 " for å logge inn eller " . BOLD . "!q register" . RESET . " for å registrere en bruker.");
   }
}


sub first_run {
   my $core = shift;
   $core->heap->{Q} ||= {}; #Create namespace.
   my %config;
   $core->module_config(\%config, $core->where);
   $core->dbi_connect({ database => $config{database}, username => $config{username}, password => $config{password}, name => 'quiz' });

}

sub Channels {
   my $core = shift;
  
   my @channels;
   foreach my $channel ( @{ $core->config->{Channel} } ) {
      my %config;
      $core->module_config(\%config, $channel);
      if($config{enabled}) {
	 push(@channels, $channel);
      }
   }
   return @channels;
}

__PACKAGE__->install({
	   events => { public => 1, nick => 1, join => 1 },
});

1;
