package Boggle::Board;

use strict;
use warnings;

use vars qw(@CHARS);

my @CHARS = qw(A A A A A A A B B C D E E E E E E E E E E E F F G G G G H H H I I I I I I J J K K K K L L L L L L M M M N N N N N N N O O O O O P P Q R R R R R R S S S S S S T T T T T T U U U V V W X Y Z);

sub new {
   my $self = shift;

   my @board_lines = @_;

   my $height = $#board_lines + 1;
   my $width = 0;
   for(@board_lines) {
      chomp;
      my $l = length($_);
      $width = $l if $l > $width;
   }

   my(@board, %chars);
   my($i, $x, $y) = (0, 0, 0);
   foreach my $row (@board_lines) {
      chomp $row;
      $board[$i] ||= 0;
      my $y = 0;
      foreach my $column (split //, $row) {
	 if($column) {
	    my $c;
	    if($column =~ /[A-Z]/) {
	       $c = $column;
	    } else {
	       $c = randChar();
	    }
	    if($chars{$c}) {
	       push @{ $chars{$c} }, $i;
	    } else {
	       $chars{$c} = [$i];
	    }
	    $board[$i] = $c;
	 } else {
	    $board[$i] = 0;
	 }
	 $i++;
      }
   }

   # @board:
   #   1 2 3
   #   4 5 6
   #   7 8 9

   #Create adjacency matrices:

   my @adj;

   for(my $n = 0; $n <= $#board; $n++) {
      next unless($board[$n]);
      my $row = int( $n / $width);
      my $col = $n % $width;
      my @n = ();
      if($row == 0) {
	 #First row.
	 if($col == 0) {
	    # X....
	    # .....
	    for($n+1, $n + $width, $n + $width + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    
	    $adj[$n] = \@n;
	 } elsif($col == $width - 1) {
	    # ....X
	    # .....
	    for($n - 1, $n + $width - 1, $n + $width) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 } else {
	    # .XXX.
	    # .....
	    for($n - 1, $n + 1, $n + $width - 1, $n + $width, $n + $width + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 }
      } elsif($row == $height - 1) {
	 #Last row.
	 if($col == 0) {
	    # .....
	    # X....
	    for($n - $width, $n - $width + 1, $n + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    
	    $adj[$n] = \@n;
	 } elsif($col == $width - 1) {
	    # .....
	    # ....X
	    for($n - $width - 1, $n - $width, $n - 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 } else {
	    # .....
	    # .XXX.
	    for($n - $width - 1, $n - $width, $n - $width + 1, $n - 1, $n + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 }
    
      } else {
	 #Middle-rows.
	  if($col == 0) {
	    # .....
	    # X....
	    # .....
	    for($n - $width, $n - $width + 1, $n + 1, $n + $width, $n + $width + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    
	    $adj[$n] = \@n;
	 } elsif($col == $width - 1) {
	    # .....
	    # ....X
	    # .....
	    for($n - $width - 1, $n - $width, $n - 1, $n + $width - 1, $n + $width) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 } else {
	    # .....
	    # .XXX.
	    # .....
	    for($n - $width - 1, $n - $width, $n - $width + 1, $n - 1, $n + 1, $n + $width - 1, $n + $width, $n + $width + 1) {
	       push(@n, $_) if($board[$_]);
	    }
	    $adj[$n] = \@n;
	 }
      }
   }
   return bless { board => \@board, adj => \@adj, height => $height, width => $width, chars => \%chars }, $self;
}

sub board { return shift->{board}; }
sub adj { return shift->{adj}; }
sub chars { return shift->{chars}; }
sub height { return shift->{height}; }
sub width { return shift->{width}; }

sub drawText {
   my $self = shift;

   my $t;
   for(my $i = 0; $i < ($self->height * $self->width);) {
      for(1 .. $self->width) {
	 if($self->board->[$i]) {
	    $t .= sprintf(" %s ", $self->board->[$i]);
	 } else {
	    $t .= "   ";
	 }
	 $i++;
      }
      $t .= "\n";
   }
   return $t;
}

sub drawHTML {
   my $self = shift;

   my $t = "<table class=\"board\">\n";
   for(my $i = 0; $i < ($self->height * $self->width);) {
      $t .= "\t<tr>\n";
      for(1 .. $self->width) {
	 $t .= sprintf("\t\t<td>%s</td>\n", $self->board->[$i] ? $self->board->[$i] : "");
	 $i++;
      }
      $t .= "\t</tr>\n";
   }
   $t .= "</table>\n";
   return $t;
}

sub debug { print @_ if 0 };

sub findWord {
   my $self = shift;
   my $word = uc(shift);
   my $start = shift;
   my @used = @_;

   return 1 unless(length($word)); #End of recursion.

   $used[$start] = $start if $start;
   debug "*** used: " . join ", ", @used . "\n" if @used;

   my $char = substr($word, 0, 1, ''); #Find one character a time..

   debug "*** $char - $word";

   unless(defined $start) {
      #First step in recursion:
      foreach my $n ( @{ $self->chars->{$char} }) {
	 debug "*** Starting from $n\n";
	 return 1 if($self->findWord($word, $n, @used));
      }
      debug "*** Giving up completely\n";
      return 0;
   }


   foreach my $a (@{ $self->adj->[$start]  }) {
      debug "*** recursing from $a\n";
      next if(defined $used[$a]); #Can't use a character twice.
      debug "\t*** $a not used.\n";
      next unless($self->board->[$a] eq $char); #Not equal.
      debug "\t*** $a matches.\n";
      return 1 if($self->findWord($word, $a, @used)); #Recurse and see if the remainder of the word can be found from where we are.
      debug "*** Recursion found nothing.\n";
   }

   debug "Giving up\n";

   return 0;
}

sub randChar {
   return @CHARS[int(rand($#CHARS))];
}

sub row {
   my $self = shift;
   my $x = shift;
   return int( $x / $self->width);
}

sub col {
   my $self = shift;
   my $x = shift;
   return $x % $self->width;
}

1;
