package Logger;

# $Id$


use base qw(Plugin);
use Common;

use Data::Dumper;

use My::DBI qw(:all);

use constant JOIN => 1;
use constant PART => 2;
use constant QUIT => 3;
use constant KICK => 4;
use constant MODE => 5;
use constant NICK => 6;
use constant MSG => 7;
use constant PRES => 8; #Present when joining.

sub public
{
   my $core = shift;
   my ($who, $where, $msg) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   dbi_insert_from_hash($core->dbh('log'), 'log', { 
	 channel => $cid,
	 nick => $nick,
	 host => $host,
	 event => MSG,
	 text => $msg,
   });

}

sub who
{
   my $core = shift;
   my ($who, $where) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   unless(dbi_exists($core->dbh('log'), "SELECT nick FROM log_presence WHERE nick=? AND host=? AND channel=?", $nick, $host, $cid)) {
	 dbi_insert_from_hash($core->dbh('log'), 'log_presence', {
	       channel => $cid,
	       nick => $nick,
	       host => $host,
	 });
   }
}

sub join
{
   my $core = shift;
   my ($who, $where) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   return if($nick eq $core->mynick);

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   dbi_insert_from_hash($core->dbh('log'), 'log', { 
	 channel => $cid,
	 nick => $nick,
	 host => $host,
	 event => JOIN,
   });

   dbi_insert_from_hash($core->dbh('log'), 'log_presence', {
	 channel => $cid,
	 nick => $nick,
	 host => $host,
   });
}

sub part
{
   my $core = shift;
   my ($who, $where) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   dbi_insert_from_hash($core->dbh('log'), 'log', { 
	 channel => $cid,
	 nick => $nick,
	 host => $host,
	 event => PART,
   });

   prepare_and_exec($core->dbh('log'), "DELETE FROM log_presence WHERE nick=? AND host=? AND channel=?", $nick, $host, $cid);

}

sub mode
{
   my $core = shift;
   my ($who, $where, $mode, @argument) = @_;
   my $argument = join ",", @argument;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   dbi_insert_from_hash($core->dbh('log'), 'log', { 
	 channel => $cid,
	 nick => $nick,
	 host => $host,
	 event => MODE,
	 text => "$mode $argument",
   });
}

sub quit
{
   my $core = shift;
   my ($who, $msg) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my @cids = dbi_exec_for_row_list($core->dbh('log'), "SELECT channel FROM log_presence WHERE nick=? AND host=?", $nick, $host);

   foreach my $cid (@cids) {
      dbi_insert_from_hash($core->dbh('log'), 'log', { 
	    channel => $cid,
	    nick => $nick,
	    host => $host,
	    event => QUIT,
	    text => $msg,
      });
   }

   prepare_and_exec($core->dbh('log'), "DELETE FROM log_presence WHERE nick=? AND host=?", $nick, $host);
}

sub kick
{
   my $core = shift;
   my ($who, $where, $kicked, $kickmsg) = @_;

   $who =~ /(.*)!(.*)/;
   my $nick = $1;
   my $host = $2;

   my $cid = $core->{Logger}->{channels}->{$where};
   return unless $cid;

   dbi_insert_from_hash($core->dbh('log'), 'log', { 
	 channel => $cid,
	 nick => $nick,
	 host => $host,
	 event => KICK,
	 text => "$kicked,$kickmsg",
   });
   prepare_and_exec($core->dbh('log'), "DELETE FROM log_presence WHERE nick=? AND host=? AND channel=?", $nick, $host, $cid);
}

sub nick
{
   my $core = shift;
   my ($who, $newnick) = @_;

   $who =~ /(.*)!(.*)/;
   my ($nick, $host) = ($1, $2);

   my @cids = dbi_exec_for_row_list($core->dbh('log'), "SELECT channel FROM log_presence WHERE nick=? AND host=?", $nick, $host);

   foreach my $cid (@cids) {
      dbi_insert_from_hash($core->dbh('log'), 'log', { 
	    channel => $cid,
	    nick => $nick,
	    host => $host,
	    event => NICK,
	    text => $newnick,
      });
      
   }
   prepare_and_exec($core->dbh('log'), "UPDATE log_presence SET nick=? WHERE nick=? AND host=?", $newnick, $nick, $host);
}

sub first_run
{
   my $core = shift;
   
   $core->{Logger} ||= {};

   my %config;
   $core->module_config(\%config);

   $core->dbi_connect({ 
	 database => $config{db}{database},
	 username => $config{db}{username},
	 password => $config{db}{password},
	 name => 'log',
   });

   my $dbh = $core->dbh('log');

   foreach my $channel (@{ $config{channels} }) {
      unless(dbi_exists($dbh, "SELECT id FROM log_channels WHERE name=?", $channel)) {
	    dbi_insert_from_hash($dbh, 'log_channels', { name => $channel } );
      }
   }
   
   my %h = dbi_exec_for_paired_hash($dbh, "SELECT name, id FROM log_channels");
   $core->{Logger}->{channels} = \%h;

   prepare_and_exec($dbh, "DELETE FROM log_presence");
}

__PACKAGE__->install({
	events => { public => 1, join => 1, part => 1, mode => 1, kick => 1, quit => 1, nick => 1, who => 1 },
});
