package Quiz::Team;
use warnings;
use strict;

use vars qw($dbh @ISA);

use Quiz::User;
use My::DBI qw(:all);

=item1 NAME

Quiz::Team

=item1 DESCRIPTION

A team is a bundle of C<Quiz::User>-objects.

=cut

=item1 SYNOPSIS

 my $team = Quiz::Team->new({ round => $r, team => 2 });

=item1 METHODS

The following methods are available:

=over 4

=item new ({ round => $r, team => 2 });

B<round> is a C<Quiz::Round>-object which must be started.

B<team> is the team ID.

=cut

sub new
{
    my $self = shift;
    my $args = shift;

    return unless($args->{round} && $args->{team} =~ /^\d+$/);
    return unless(ref($args{round}) eq 'Quiz::Round');

    my @teams;

    foreach my $id (dbi_exec_for_column_list($dbh, "SELECT user_id FROM round_teams WHERE round=? AND team=?", $args->{round}->id, $args->{team})) {
       push(@teams, Quiz::User->GetByID($id));
    }

    return bless { teams => \@teams }, $self;
    
}

sub teams { return shift->{teams}; }

#Return the size of a team given the number of participants.
#TODO - write a nice and dandy prime factor algorithm. I doubt
#there'll be more than 25 simultaneous players in my channel
sub TeamSize
{
   shift;
   my %ts = (1,0 ,2,0 ,3,0 ,4,2 ,5,0 ,6,3 ,7,0 ,8,4 ,9,3 ,10,5 ,11,0 ,12,4 ,13,0 ,14,7 ,15,3 ,16,4 ,17,0 ,18,6 ,19,0 ,20,5
             ,21,7 ,22,11 ,23,0 ,24,6 ,25,5);
   
   return $ts(shift);
}

1;
