package Quiz::User;
use warnings;
use strict;

use vars qw(@ISA);

use User;
use base qw(User); #Subclass User.

use Digest::MD5 qw(md5_hex);
use Locale::gettext;
use POSIX;
setlocale(LC_MESSAGES, "");
bindtextdomain('erwin', 'locale/');
textdomain('erwin');

use My::DBI qw(:all);

sub numberOfQuestions
{
    my $self = shift;
    return dbi_exec_for_scalar($self->dbh, "SELECT count(id) FROM questions WHERE author = ? AND active = 't'", $self->id);
}

sub numberOfTaggedQuestions
{
    my $self = shift;
    return dbi_exec_for_scalar($self->dbh, "SELECT count(id) FROM questions WHERE author = ? AND tagged = 't'", $self->id);
}

sub numberOfDisabledQuestions
{
    my $self = shift;
    return dbi_exec_for_scalar($self->dbh, "SELECT count(id) FROM questions WHERE author = ? AND active = 'f'", $self->id);
}

sub stats
{	
   my $self = shift;
   my $key = shift;

   my %h = dbi_exec_for_row_hash($self->dbh, "SELECT * FROM userinfo WHERE id = ?",  $self->id);
   return \%h;
}

sub scores
{	
   my $self = shift;
   my $key = shift;

   my %h = dbi_exec_for_row_hash($self->dbh, "SELECT * FROM scores WHERE id = ?",  $self->id);
   return \%h;
}


sub Register 
{
   my $core = shift;
   return if($core->testing);
  
   my $User = User->GetByHostmask($core->dbh('users'), $core->heap->{who});
   return sprintf("Du har allerede en bruker. (\"%s\").", $User->username) if($User);

   $core->startConversation($core->heap->{nick}, 'register_username', 'Skriv inn et brukernavn mellom fire og ni tegn. (Kun bokstaver og tall. Store bokstaver konverteres til små.) Skriv inn "!avbryt" når som helst for å avbryte.');
   return;
}

sub register_username 
{
   my $core = shift;
   my ($data, $msg) = @_;

   return undef if($msg =~ /!avbryt/);

   if($msg !~ /^\w{4,9}$/) {
      $core->cmsg("'$msg' er ugyldig. Skriv inn et brukernavn med KUN bokstaver og tall, som er på minst fire og maks ni tegn.");
      return 'register_username';
   } elsif(User->GetByName($core->dbh('quiz'), lc($msg))) {
      $core->cmsg("Brukernavnet '$msg' er tatt. Velg et annet.");
      return 'register_username';
   } else {
      $data->{username} = $msg;
      $core->cmsg("Skriv inn et passord ulikt brukernavnet på minst 4 tegn. Husk at IRC er et usikkert medium, så ikke bruk viktige passord.");
      return 'register_password';
   }
}

sub register_password 
{
   my $core = shift;
   my ($data, $msg) = @_;

   return undef if($msg =~ /!avbryt/);

   if($msg !~ /^.{4,}$/) {
      $core->cmsg("Jeg sa at passordet skulle være på MINST fire tegn. Skriv et lengre passord, din bøtteknott.");
      return 'register_password';
   } elsif(lc($msg) eq lc($data->{username})) {
      $core->cmsg("Er du døvlest? Skriv inn et passord som er ULIKT brukernavnet.");
      return 'register_password';
   } else {
      my $User = Quiz::User->new($core->dbh('quiz'), lc($data->{username}));
      $User->password($msg);
      $User->hostmask($core->heap->{who});
      $User->save;
      foreach my $channel( Quiz::Channels($core) ) {
         $core->setMode($channel,'+v', $core->heap->{nick});
      }
      $core->cmsg("Brukeren er laget. Velkommen.");
      return undef;
   }
}

sub Auth {
   my ( $core, $backrefs, $args ) = @_;
   return if($core->testing);
   my ($username, $password ) = split(/ /, $backrefs->[0], 2);
   
   if($username && $password) {
      my $User = User->GetByName($core->dbh('users'), $username);
      return gettext('Invalid credentials.') unless($User && md5_hex($password) eq $User->password);

      my $oldhostmask = $User->hostmask;
      $oldhostmask =~ /(.*)!/;
      my $oldnick = $1;

      $User->setHostmask($core->heap->{who});
      $core->isInfo;

      foreach my $channel( Quiz::Channels($core) ) {
	 unless($oldnick eq $core->heap->{nick}) {
            $core->setMode($channel,'-v', $oldnick);
	 }
         $core->setMode($channel,'+v', $core->heap->{nick});
      }

      return gettext('Logged in.');
   } elsif($username) {
      my $User = User->GetByName($core->dbh('users'), $username);
      if($User) {
         $core->startConversation($core->heap->{nick}, 'login_password', gettext('Please enter your password.'), { user => $User });
      } else {
         return gettext('No user with that name exists.');
      }
   } else {
       $core->startConversation($core->heap->{nick}, 'login_username', gettext('Please enter your username.'));
   }
}

sub login_username {
   my ($core, $data, $msg) = @_;

   if($msg =~ /\W/) {
      $core->cmsg(gettext('Invalid characters in username.'));
      return undef;
   }

   my $User = User->GetByName($core->dbh('users'), $msg);
   if($User) {
      $data->{user} = $User;
      $core->cmsg(gettext('Please enter your password.'));
      return 'login_password';
   } else {
      $core->cmsg(gettext('No user with that name exists.'));
      return undef; #End of conversation.
   }
}

sub login_password {
   my ($core, $data, $msg) = @_;

   my $User = $data->{user};

   if($User->password eq md5_hex($msg)) {
      $User->setHostmask($core->heap->{who});
      foreach my $channel( Quiz::Channels($core) ) {
         $core->setMode($channel,'+v', $core->heap->{nick});
      }
      $core->cmsg(gettext('Logged in.'));
   } else {
      $core->cmsg(gettext('Invalid credentials.'));
   }

   return undef; #End of conversation.
}

sub Logout {
    my ( $core, $backrefs, $args ) = @_;
    return if($core->testing);

    my $user = User->GetByHostmask($core->dbh('users'), $core->heap->{who});

    if($user) {
       $user->hostmask('-');
       $user->save;
       $core->isInfo;

      foreach my $channel( Quiz::Channels($core) ) {
         $core->setMode($channel,'-v', $core->heap->{nick});
      }

       return gettext('Logged out.');
    } else {
       return gettext('Not logged in.');
    }
}

1;
