package Statistics;
# $Id$
# Gather various statistics, using RRD.

use strict;
use base qw(Plugin);
use RRDs;

use vars qw(%RRDS @TYPES);

use constant COUNTER	=> 0;
use constant DERIVE	=> 1;
use constant ABSOLUTE	=> 2;
use constant GAUGE	=> 3;

@TYPES = qw(COUNTER DERIVE ABSOLUTE GAUGE);

%RRDS = (
   msgs => {
      type => GAUGE,
      DS => ['lines','questions'],
   },
   clients => {
      type => GAUGE,
      DS => ['clients'],
   },
#   smileys => {
#      type => GAUGE,
#      DS => ['happy','sad'],
#   },
);

sub cron
{
   my $core = shift;

   $core->addTick(time() + 300);

   #Update RRDs..
   my %config;
   $core->module_config(\%config);
   return unless @{ $config{channels} };

   foreach my $channel (@{ $config{channels} }) {
      RRDs::update("$config{datadir}/${channel}_clients.rrd", "N:" . scalar(keys %{ $core->hostmasks->{$channel} }));
      
      RRDs::update("$config{datadir}/${channel}_msgs.rrd",
	 "N:" . $core->heap->{Statistics}->{$channel}->{lines} .
          ":" . $core->heap->{Statistics}->{$channel}->{questions}
       );

      $core->heap->{Statistics}->{$channel}->{lines} = 0;
      $core->heap->{Statistics}->{$channel}->{questions} = 0;
   }
}

sub public
{
   my ($core, $who, $where, $msg) = @_;
   $core->heap->{Statistics}->{$where}->{lines}++;
   $core->heap->{Statistics}->{$where}->{questions}++ if($msg =~ /\?/);
}

sub first_run
{
   my $core = shift;

   $core->heap->{Statistics} ||= {};

   my %config;
   $core->module_config(\%config);
   return unless @{ $config{channels} };

   $config{datadir} ||= '.';


   #Make RRDs
   foreach my $channel (@{ $config{channels} }) {
      foreach my $rrd (keys %RRDS) {
	 my $filename = "$config{datadir}/${channel}_$rrd.rrd";
	 if(! -e $filename) {
	    my (@DS, @RRA);
	    foreach my $ds (@{ $RRDS{$rrd}{DS} }) {
	       #E.g: "DS:questions:GAUGE:300:0:U"
	       push (@DS, join ":", 'DS', $ds, $TYPES[$RRDS{$rrd}{type}], 600, 'U', 'U');
	    }

	    push (@RRA, join ":", "RRA", 'LAST', '0.5', 1, 576);	#2 days of 5 minute averages
	    push (@RRA, join ":", "RRA", 'LAST', '0.5', 6, 672);	#2 weeks of 1/2 hour averages
	    push (@RRA, join ":", "RRA", 'LAST', '0.5', 24, 732);	#2 months of 2 hour averages
	    push (@RRA, join ":", "RRA", 'LAST', '0.5', 144, 1460);	#2 years of 12 hour averages

	    #Create it.
	    if( RRDs::create($filename,
	       "--start",  time(),
	       "--step", 300,
	       @DS,
	       @RRA
	    )) {
	    } else {
	       print RRDs::error;
	    }
	 }
      }
   }

   #Get ticker running. :)
   $core->addTick(time() + 10);
}

__PACKAGE__->install({
	events => { public => 1 },
	immutable => 1,
});

1;
